import 'package:flutter/material.dart';
import 'package:bmi_global/utils/parsers.dart';
import 'package:intl/intl.dart';

class Dates {
  static List<DateTime> fromRange(DateTimeRange dateTimeRange) {
    final start = dateTimeRange.start;
    final end = dateTimeRange.end;
    final days = <DateTime>[];
    for (int i = 0; i <= end.difference(start).inDays; i++) {
      days.add(start.add(Duration(days: i)));
    }
    return days;
  }

  static List<DateTime> between(DateTime start, DateTime end) {
    switch (start.compareTo(end)) {
      case 1:
        throw Error.throwWithStackTrace(
          "start can't be greator then end",
          StackTrace.current,
        );
      case 0:
        return [];
    }
    final days = <DateTime>[];
    for (int i = 0; i <= end.difference(start).inDays; i++) {
      days.add(start.add(Duration(days: i)));
    }
    return days;
  }
}

enum TimeStampFormate {
  /// yyyy-MM-ddTHH:mm:ss** formate
  isoString,

  /// yyyy-MM-dd** formate
  yyyyMMdd,

  /// yyyy-MM** formate
  yyyyMM
}

extension on TimeStampFormate {
  get hasDate {
    switch (this) {
      case TimeStampFormate.isoString:
        return true;
      case TimeStampFormate.yyyyMMdd:
        return true;
      case TimeStampFormate.yyyyMM:
        return false;
    }
  }

  get hasTime {
    switch (this) {
      case TimeStampFormate.isoString:
        return true;
      case TimeStampFormate.yyyyMMdd:
        return false;
      case TimeStampFormate.yyyyMM:
        return false;
    }
  }
}

class TimeStampString {
  final String value;
  final DateTimePlus dateTimePlus;
  final TimeStampFormate formate;

  DateTime get dateTime => dateTimePlus.dateTime;

  TimeStampString.fromDateTime(
    DateTime dateTime, [
    this.formate = TimeStampFormate.isoString,
  ])  : dateTimePlus = DateTimePlus.fromDateTime(dateTime),
        value = dateTime.toIso8601String().substring(
            0,
            formate.hasTime
                ? null
                : formate.hasDate
                    ? 10
                    : 7);

  TimeStampString.fromString(
    this.value, [
    this.formate = TimeStampFormate.isoString,
  ]) : dateTimePlus = formate.hasTime
            ? DateTimePlus(
                toInt(value.substring(0, 4)),
                toInt(value.substring(5, 7)),
                toInt(value.substring(8, 10)),
                toInt(value.substring(11, 13)),
                toInt(value.substring(14, 16)),
                toInt(value.substring(17, 19)),
              )
            : formate.hasDate
                ? DateTimePlus(
                    toInt(value.substring(0, 4)),
                    toInt(value.substring(5, 7)),
                    toInt(value.substring(8, 10)),
                  )
                : DateTimePlus(
                    toInt(value.substring(0, 4)),
                    toInt(value.substring(5, 7)),
                  );

  String formateTime({bool inAmPm = true, bool withSec = false}) {
    try {
      final hr = dateTime.hour;
      final min = dateTime.minute;
      final sec = dateTime.second;
      if (inAmPm) {
        final isPm = hr > 12;
        return "${isPm ? (hr - 12) : hr}:$min${withSec ? ":$sec" : ""}  ${isPm ? "pm" : "am"}";
      }
      return "@ $hr:$min${withSec ? ":$sec" : ""}";
    } catch (_) {
      return "@ ${"".padLeft(withSec ? 8 : 5)}";
    }
  }

  String formateDate({
    bool name = true,
    bool withYear = true,
    bool withDate = true,
  }) {
    try {
      if (name) {
        return dateTimePlus.toDateString(
          inName: true,
          withYear: withYear,
          withDate: withDate,
        );
      }
      return "Date: ${value.substring(0, 10).substring(withYear ? 0 : 5, withDate ? null : 7)}";
    } catch (_) {
      return "Date: ${"".padLeft((withYear ? 5 : 0) + 2 + (withDate ? 3 : 0))}";
    }
  }
}

class DateTimePlus {
  final DateTime dateTime;
  const DateTimePlus.fromDateTime(this.dateTime);
  DateTimePlus(
    int year, [
    int? month,
    int? day,
    int? hour,
    int? minute,
    int? second,
    int? millisecond,
    int? microsecond,
  ]) : dateTime = DateTime(
          year,
          month ?? 1,
          day ?? 1,
          hour ?? 0,
          minute ?? 0,
          second ?? 0,
          millisecond ?? 0,
          microsecond ?? 0,
        );

  static final _formate = DateFormat('yyyy-MM-dd');
  static final _nameFormate = DateFormat('dd MMM yyyy');

  String toDateString({
    bool withYear = true,
    bool inName = false,
    bool withDate = true,
  }) {
    if (inName) {
      return _nameFormate
          .format(dateTime)
          .substring(withDate ? 0 : 3, withYear ? null : 6);
    }
    return _formate
        .format(dateTime)
        .substring(withYear ? 0 : 5, withDate ? null : 7);
  }

  DateTime get firstDayOfMonth => DateTime(dateTime.year, dateTime.month);
  DateTime get lastDayOfMonth =>
      DateTime(dateTime.year, dateTime.month + 1, -1, 24);
}

class IntMoney {
  final int money;

  static const _rs = "₹ ";
  static final _amount = NumberFormat("##,##,##,###.##", "en_IN");
  const IntMoney(this.money);

  String displayStr({bool trail = true, bool lead = true}) {
    var val = "";
    if (lead) val += _rs;
    val += _amount.format(money / 1000);
    if (trail) val += "/-";
    return val;
  }
}

class IntQuntity {
  final int quntity;

  static final _quntity = NumberFormat("##,##,##,###.###", "en_IN");
  const IntQuntity(this.quntity);

  String toQuntity() {
    return _quntity.format(quntity / 1000);
  }
}
